mips-tdep.c (heuristic_proc_start): Recognise MIPS16 "save"

Maciej W. Rozycki macro@mips.com
Fri Jul 20 17:18:00 GMT 2007


Hello,

 The MIPS16e ASE adds "save" to the list of MIPS16 instructions, which is 
typically used in a function's prologue.  It is therefore useful to have 
it recognised by our heuristic analyser.

 Tested for mipsisa32-sde-elf, with the mips-sim-sde32/-mips16/-EB and 
mips-sim-sde32/-mips16/-EL target boards with no regressions.

2004-07-20  Chris Dearman  <chris@mips.com>

	* mips-tdep.c (heuristic_proc_start): Add more MIPS16 function
	prologue instructions.

 OK to apply?

  Maciej

12271-6.diff
Index: binutils-quilt/src/gdb/mips-tdep.c
===================================================================
--- binutils-quilt.orig/src/gdb/mips-tdep.c	2007-05-22 19:11:41.000000000 +0100
+++ binutils-quilt/src/gdb/mips-tdep.c	2007-05-22 19:11:51.000000000 +0100
@@ -2363,15 +2363,28 @@
 
 	/* On MIPS16, any one of the following is likely to be the
 	   start of a function:
+  	   extend save
+	   save
 	   entry
 	   addiu sp,-n
 	   daddiu sp,-n
 	   extend -n followed by 'addiu sp,+n' or 'daddiu sp,+n'  */
 	inst = mips_fetch_instruction (start_pc);
-	if (((inst & 0xf81f) == 0xe809 && (inst & 0x700) != 0x700)	/* entry */
-	    || (inst & 0xff80) == 0x6380	/* addiu sp,-n */
-	    || (inst & 0xff80) == 0xfb80	/* daddiu sp,-n */
-	    || ((inst & 0xf810) == 0xf010 && seen_adjsp))	/* extend -n */
+	if ((inst & 0xff80) == 0x6480)		/* save */
+	  {
+	    if (start_pc - instlen >= fence)
+	      {
+		inst = mips_fetch_instruction (start_pc - instlen);
+		if ((inst & 0xf800) == 0xf000)	/* extend */
+		  start_pc -= instlen;
+	      }
+	    break;
+	  }
+	else if (((inst & 0xf81f) == 0xe809
+		  && (inst & 0x700) != 0x700)	/* entry */
+		 || (inst & 0xff80) == 0x6380	/* addiu sp,-n */
+		 || (inst & 0xff80) == 0xfb80	/* daddiu sp,-n */
+		 || ((inst & 0xf810) == 0xf010 && seen_adjsp))	/* extend -n */
 	  break;
 	else if ((inst & 0xff00) == 0x6300	/* addiu sp */
 		 || (inst & 0xff00) == 0xfb00)	/* daddiu sp */



More information about the Gdb-patches mailing list